package com.wyattjmiller.routes

import com.wyattjmiller.entities.AuthorDraft
import com.wyattjmiller.repositories.AuthorDao
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*

class AuthorRoute {
    companion object {
        private val author = AuthorDao()

        private fun Route.getAllAuthors() {
            get("/author") {
                call.respond(author.getAllAuthors())
            }
        }

        private fun Route.getAuthor() {
            get("/author/{id?}") {
                val id = call.parameters["id"]?.toInt() ?: return@get call.respond(
                    HttpStatusCode.BadRequest, "Missing author identifier :("
                )

                val res = author.getAuthor(id) ?: return@get call.respond(
                    HttpStatusCode.NotFound, "No recipe found :("
                )

                call.respond(res)
            }
        }

        private fun Route.createAuthor() {
            post("/author") {
                val req = call.receive<AuthorDraft>()
                val res = author.createAuthor(req)
                call.respond(HttpStatusCode.Created)
            }
        }

        private fun Route.updateAuthor() {
            put("/author/{id?}") {
                val req = call.receive<AuthorDraft>()
                val id = call.parameters["id"]?.toIntOrNull() ?: return@put call.respond(
                    HttpStatusCode.BadRequest, "Missing author identifier :("
                )

                if (author.updateAuthor(id, req)) {
                    call.respond(HttpStatusCode.Created)
                } else {
                    call.respond(
                        HttpStatusCode.NotFound, "No author found :("
                    )
                }
            }
        }

        fun Application.authorRoutes() {
            routing {
                getAllAuthors()
                getAuthor()
                createAuthor()
                updateAuthor()
            }
        }
    }
}