package com.wyattjmiller.routes

import com.typesafe.config.ConfigFactory
import com.wyattjmiller.auth.JwtConfig
import com.wyattjmiller.entities.UserDraft
import com.wyattjmiller.entities.UserLogin
import com.wyattjmiller.repositories.UserDao
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.config.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*

class UserRoute {
    companion object {
        private val user = UserDao()
        private val tokenManager = JwtConfig(HoconApplicationConfig(ConfigFactory.load()))

        private fun Route.login() {
            post("/login") {
                val userCreds = call.receive<UserLogin>()
                val usernameResult = user.usernameCheck(userCreds)
                val passwordResult = user.passwordCheck(userCreds)

                if (usernameResult == null || passwordResult == null) {
                    call.respond(HttpStatusCode.BadRequest)
                }

                //if (!BCrypt.checkpw(password, res?.password)) {
                //    call.respond(HttpStatusCode.BadRequest)
                //    return@post
                //}

                val token = tokenManager.generateJwtToken(userCreds)
                call.respond(HttpStatusCode.OK, token)
            }
        }

        private fun Route.getAllUsers() {
            get("/user") {
                call.respond(user.getAllUsers())
            }
        }

        private fun Route.getUser() {
            authenticate {
                get("/user/{id?}") {
                    val id = call.parameters["id"]?.toInt() ?: return@get call.respond(
                        HttpStatusCode.BadRequest, "Missing user identifier :("
                    )

                    val res = user.getUser(id) ?: return@get call.respond(
                        HttpStatusCode.NotFound, "No recipe found :("
                    )

                    call.respond(res)
                }
            }
        }

        private fun Route.createUser() {
            post("/user") {
                val req = call.receive<UserDraft>()
                val res = user.createUser(req)
                call.respond(res)
            }
        }

        private fun Route.updateUser() {
            authenticate {
                put("/user/{id?}") {
                    val req = call.receive<UserDraft>()
                    val id = call.parameters["id"]?.toIntOrNull() ?: return@put call.respond(
                        HttpStatusCode.BadRequest, "Missing author identifier :("
                    )

                    if (user.updateUser(id, req)) {
                        call.respond(HttpStatusCode.Accepted)
                    } else {
                        call.respond(
                            HttpStatusCode.NotFound, "No author found :("
                        )
                    }
                }
            }
        }

        fun Application.userRoutes() {
            routing {
                login()
                getUser()
                createUser()
                updateUser()
            }
        }
    }
}